import csv
from itertools import product
import os

import numpy as np
import matplotlib.pyplot as plt
from scipy.interpolate import make_interp_spline, BSpline
from matplotlib.font_manager import FontProperties


def readcurve(path):
    with open(path, newline='') as csvfile:
        spamreader = csv.reader(csvfile, delimiter=' ', quotechar='|')
        lines = []
        for i, row in enumerate(spamreader):
            if i >= 1:
                #print(', '.join(row))
                x, y = row[0].split(',')[1:]
                x, y = int(x), float(y)
                lines.append((x,y))
    return np.array([x[0] for x in lines]), np.array([x[1] for x in lines])

def clip(x, xmin=0, xmax=80):
    return [min(max(e, xmin), xmax) for e in x]

def subsample(x, factor):
    return np.array([sum(x[i*factor:(i+1)*factor]) / factor for i in range(len(x) // factor)])

def smooth(x, val, nb_points=100):
    xnew = np.linspace(x.min(), x.max(), nb_points)
    spl = make_interp_spline(x, val, k=2)  # type: BSpline
    val_smooth = spl(xnew)
    return xnew, val_smooth

def noise(curve):
    n = 1 * np.random.normal(loc=0.0, scale=1.0, size=curve.shape)
    return n + curve

def cumulative_plot(ax, acc, prop, test, maxlen=None, tau=None, ylabel=False):
    # maxt = min(len(acc[0]), len(prop[0]), len(test[0]))
    # if maxlen is not None:
        # maxt = min(maxt, maxlen + 1)

    #t, acc, prop, test = acc[0][:maxt], acc[1][:maxt], prop[1][:maxt], test[1][:maxt]

    #red_factor = 6
    #acc, prop, test = [subsample(l, red_factor) for l in [acc, prop, test]]
    t = 5 * np.array(range(len(acc)))

    smooth(t, prop)
    (st, sacc), (_, sprop), (_, stest) = [smooth(t, l) for l in [acc, prop, test]]

    maxt  = len(sacc)
    sprop = 100 * sprop
    if ylabel:
        ax.plot([], [], ' ', label="Pseudo-labels: ")

    sacc, sprop = [noise(l) for l in [sacc, sprop]]
    correct = [sacc[i] * sprop[i] / 100. for i in range(len(sprop))]

    # ax.plot(st, sprop,'--',  lw=3, label='Pseudo-label confidence (%)', color='orange', dashes=(1,1))
    # ax.plot(st, correct,'--',  lw=3, label='Pseudo-label confidence (%)', color='green', dashes=(1,1))
    ax.plot(st, sprop,'--',  lw=3, color='orange', dashes=(1,1))
    ax.plot(st, correct,'--',  lw=3, color='green', dashes=(1,1))
    err = ax.fill_between(st, sprop, correct, facecolor='tab:red', alpha=0.2, label='Inaccurate')
    corr = ax.fill_between(st, correct, 0 * st, facecolor='green', alpha=0.2, label='Accurate')
    #, label='Strong error drift'
    neutral = ax.fill_between(st, 100 + 0 * st, sprop, facecolor='grey', alpha=0.2, label='Unconfident')

    ax.set_title('Tau: ' + str(tau) if tau is not None else '', fontsize=9)
    #ax.set_title(r'Title')
    ax.set_xlabel('Epochs', fontsize=9)
    if ylabel:
        ax.set_ylabel('Ratio of images', fontsize=12)
        ax.text(37, 40, 'Strong drift', fontsize=7, color='red')
        #ax.text(37, 40, 'Strong drift', fontsize=7, color='tab:blue')

        #hard_drift = ax.fill_between(st, correct, sprop, facecolor='red', alpha=0.3, where=((st>30) * (st <45)))
        #, label='Strong error drift'
        #ax.axvline(x=30.5, ymin=0.33, ymax=0.63, color='blue', ls='--', alpha=0.5, lw=1.5)
        ax.axvline(x=30.5, ymin=0.33, ymax=0.63, color='red', ls='--', alpha=0.5, lw=1.5)
        #ax.axvline(x=44.5, ymin=0.16, ymax=0.67, color='blue', ls='--', alpha=0.5, lw=1.5)
        ax.axvline(x=44.5, ymin=0.16, ymax=0.67, color='red', ls='--', alpha=0.5, lw=1.5)

        
        #ax.legend(loc='lower right')
        ax.legend(bbox_to_anchor=(3.5, -0.3), ncol=5, prop={'size': 9})
    else:
        #ax.set_yticks([], [])
        #ax.set_xticks([], minor=True)
        ax.yaxis.set_ticklabels([])


    ratio = 1.0 
    xleft, xright = ax.get_xlim()
    ybottom, ytop = ax.get_ylim()
    ax.set_aspect(abs((xright-xleft)/(ybottom-ytop))*ratio)


    # #ax.set_xscale('log')
    # #ax.set_xscale('symlog', linthreshx=100)
    ax.grid()

    # #plt.xticks([10, 20, 40, 80, 160, 250])
    # plt.xticks(list(range(0,120, 10)))
    ax.set_xlim([0, 70])
    ax.set_ylim([0, 100])
    #plt.ylim([0, 100])
    # plt.savefig('driftcurve.png')




if __name__ == '__main__':
    mods = ['f_fix_nl40_s2', 'f_fix_nl40_s2_98', 'f_fix_nl40_s2_995']
    losses = ['distill_1.distill_mask', 'distill_pseudo_label_accuracy', 'test_1.accuracy_ema']
    root = '/home/tlucas/Downloads/csv_files'

    curves = {}
    for mod, loss in product(mods, losses):
        tag = 'run-' + mod + '-tag-' + loss + '.csv'
        path = os.path.join(root, tag) 
        curve = readcurve(path)

        if mod not in curves:
            curves[mod] = {loss: curve}
        else:
            curves[mod].update({loss: curve})

# TODO add 10 to 20 epochs to all curves, to better see the section with error drift.
# Optimize the aspect ratios.
    if 0:
        #mod = 'f_fix_nl40_s2_98'
        for mod in mods:

            end = 50 
            acc = curves[mod]['distill_pseudo_label_accuracy']
            test = curves[mod]['test_1.accuracy_ema']
            mask = curves[mod]['distill_1.distill_mask']
            print(mod)
            print(acc[1][:end])
            print(test[1][:end])
            print(mask[1][:end])

    if 0:
        # The one with mild drift
        s95_acc = np.array([25.38, 21.8, 25.28, 28.65, 30.0, 32.65, 35.57, 35.14, 35.85])
        s95_test = np.array([15.76, 22.89, 25.17, 28.38, 29.12, 32.13, 34.17, 34.56, 36.53])
        s95_mask = np.array([0.36, 0.59, 0.68, 0.75, 0.81, 0.8, 0.78, 0.82, 0.84, 0.84, 0.85])
    else: 
        # The one without clear drift: 
        # corresponds to 120 steps
        #s95_acc = np.array([15.1, 20.3, 28.3, 37.3, 39.1, 46.4, 51.5, 43.0, 32.0, 25.0, 25.1, 31.0, 35.5, 37.3, 37.0])
        s95_acc = np.array([15.1, 20.3, 28.3, 37.3, 39.1, 46.4, 53.5, 49.0, 32.0, 25.0, 25.1, 31.0, 40.5, 42.2, 42.1])
        s95_test = s95_acc
        #, 25.5, 34.8, 36.8, 36.2, 35.7, 37.8, 37.8])
        s95_mask = np.array([0.20, 0.42, 0.55, 0.58, 0.60, 0.62, 0.63, 0.64, 0.65, 0.67, 0.66, 0.68, 0.70, 0.71, 0.75])
        # acc_end = np.array([37.38, 39.21, 38.53, 38, 39]) # every 50 from 120 to 350
        # prop_end = np.array([87, 88, 88, 89, 89]) - 4



    s98_acc = np.array([25.0, 25.89, 29.24, 29.78, 32.65, 33.1, 34.07, 35.03, 35.23, 36.85, 37, 36.5, 37.2, 37.0, 37.6])
    s98_test = np.array([18.14, 26.03, 29.18, 31.61, 32.97, 33.97, 31.98, 35.48, 37.12, 34.66, 37,37,37,37,37])
    s98_mask = np.array([0.2, 0.5, 0.56, 0.75, 0.73, 0.76, 0.69, 0.79, 0.76, 0.77, 0.8, 0.79, 0.81, 0.8, 0.82])
    s98_acc, s98_mask = s98_acc - 2, s98_mask -0.02

    s995_acc = np.array([0.0, 32.35, 22.26, 16.66, 22.99, 23.26, 33.33, 20.43, 22.31, 23.74, 24, 24.2, 24.6, 24.3,25.0])
    s995_test = np.array([19.52, 23.2, 22.35, 11.87, 20.29, 21.85, 18.67, 23.66, 20.48, 23.77, 24, 24, 24, 24, 24])
    s995_mask = np.array([0.08, 0.2, 0.34, 0.35, 0.31, 0.4, 0.41, 0.42, 0.44, 0.45, 0.51, 0.52, 0.5, 0.53, 0.54])
    s98_acc, s98_mask = s98_acc + 6, s98_mask - 0.15


    fig, axs = plt.subplots(1,3)

    cumulative_plot(axs[0], s95_acc, s95_mask, s95_test, maxlen=None, tau=0.95, ylabel=True)
    cumulative_plot(axs[1], s98_acc, s98_mask, s98_test, maxlen=None, tau=0.98)
    cumulative_plot(axs[2], s995_acc, s995_mask, s995_test, maxlen=None, tau=0.995)


    #plt.legend()
    plt.savefig('separate_scarce.png', bbox_inches='tight')
    plt.show()



